==> atmos-2.96.1.tar.gz: atmos-2.96.1/src/params.h <==
/*
    $Id: params.h 53 2009-11-01 18:03:28Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PARAMS_H
#define _PARAMS_H

#include <algorithm>
#include <string>
#include <exception>
#include <stdexcept>

#include <boost/numeric/ublas/vector.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>

#include "coordinates.h"

using namespace boost::numeric;
// use ublas::vector for ublas vector and std::vector for std vector

/**
  @brief Container for handling parameters extracted from MASS file.
*/
template<class ContainerType> class parameters_container {
public:
	typedef ContainerType storage_type;
	typedef typename storage_type::key_type   key_type;
	typedef typename storage_type::value_type value_type;
private:
	storage_type m_storage;
public:
	parameters_container() {}
	~parameters_container() {}

	bool key_equal( const key_type& k1, const key_type& k2 ) {
		return !m_storage.key_comp()( k1, k2 ) && !m_storage.key_comp()( k2, k1 );
	}
	bool value_equal( const value_type& v1, const value_type& v2 ){
		return !m_storage.value_comp()( v1, v2 ) && !m_storage.value_comp()( v2, v1 ) ;
	}

	bool has(const typename storage_type::key_type& key) const {
		typename storage_type::const_iterator it = m_storage.find( key );
		return ( it != m_storage.end() );
	}
	const typename storage_type::mapped_type& at(const typename storage_type::key_type& key) const {
		typename storage_type::const_iterator it = m_storage.find( key );
		if( it != m_storage.end() )
			return it->second;
		throw std::range_error( std::string("Not found value for key \"") + key + std::string("\""));
	}
	typename storage_type::mapped_type& operator[] (const typename storage_type::key_type& key) {
		return m_storage[key];
	}
	template<class Y> Y as(const typename storage_type::key_type& key) const {
		return boost::lexical_cast<Y>( at(key) );
	}
	template<class Y> Y as(const typename storage_type::key_type& key, const Y& def ) const {
		if( has( key ) )
			return boost::lexical_cast<Y>( at(key) );
		return def;
	}
};

/**
  @brief Proxy for access to parameters extracted from MASS file and stored in container.
*/
template<class ParamType> class parameters_proxy {
public:
	typedef ParamType params_type;
private:
	const params_type& m_container;
public:
	parameters_proxy( const params_type& container ): m_container( container ) {}
	~parameters_proxy() {}

	std::string sitename() const { return m_container.at("general/site/sitename"); }
	Hour   longitude()     const { return Hour( m_container.at("general/site/longitude") ); }
	Degree latitude()      const { return Degree( m_container.at("general/site/latitude") ); }
	//FIXME: leak here
	Geo*   site()          const { return new Geo( longitude(), latitude() ); }

	double accumtime()     const { return m_container.template as<double>("operations/normal mode/accumtime"); }
	double exposition()    const { return m_container.template as<double>("operations/normal mode/exposition"); }

	double magnification() const { return 0.1*m_container.template as<double>("optics/common/magnification"); }

	//FIXME: make function with wildcards
	ublas::vector<double> inner_diam()    const {
		ublas::vector<double> ret(4);
		ret(0) = m_container.template as<double>("optics/segment a/inner");
		ret(1) = m_container.template as<double>("optics/segment b/inner");
		ret(2) = m_container.template as<double>("optics/segment c/inner");
		ret(3) = m_container.template as<double>("optics/segment d/inner");
		return ret;
	}
	ublas::vector<double> outer_diam()    const {
		ublas::vector<double> ret(4);
		ret(0) = m_container.template as<double>("optics/segment a/outer");
		ret(1) = m_container.template as<double>("optics/segment b/outer");
		ret(2) = m_container.template as<double>("optics/segment c/outer");
		ret(3) = m_container.template as<double>("optics/segment d/outer");
		return ret;
	}
	ublas::vector<double> nonlinearity() const {
		std::vector< std::string > ctr( counters() );
		ublas::vector<double> ret( ctr.size() );
		for( typename std::vector< std::string >::const_iterator it = ctr.begin(); it != ctr.end(); ++it ) {
			ret( it - ctr.begin() ) = m_container.template as<double>( (boost::format("Devices/Bicounter %1%/NonLinearity%2%") % (*it)[0] % (*it)[1]).str() );
		}
		return ret;
	}
	ublas::vector<double> nonpoisson() const {
		std::vector< std::string > ctr( counters() );
		ublas::vector<double> ret( ctr.size() );
		for( typename std::vector< std::string >::const_iterator it = ctr.begin(); it != ctr.end(); ++it ) {
			ret( it - ctr.begin() ) = m_container.template as<double>( (boost::format("Devices/Bicounter %1%/NonPoisson%2%") % (*it)[0] % (*it)[1]).str() );
		}
		return ret;
	}
	ublas::vector<double> scatter() const {
		ublas::vector<double> ret( 4 );
		ret(0) = m_container.template as<double>("channels/scatter/channela",0.0);
		ret(1) = m_container.template as<double>("channels/scatter/channelb",0.0);
		ret(2) = m_container.template as<double>("channels/scatter/channelc",0.0);
		ret(3) = m_container.template as<double>("channels/scatter/channeld",0.0);
		return ret;
	}
	//FIXME: use tuple
	std::vector< std::string > counters() const {
		std::vector< std::string > ret;
		ret.resize(4);
		ret[0] = m_container.at("channels/configuration/channela");
		ret[1] = m_container.at("channels/configuration/channelb");
		ret[2] = m_container.at("channels/configuration/channelc");
		ret[3] = m_container.at("channels/configuration/channeld");
		for( typename std::vector< std::string >::iterator it = ret.begin(); it != ret.end(); ++it ){
			//FIXME: use regex
			if( it->size() < 10 || it->substr(0,7) != "Counter" || ( (*it)[8] != '1' && (*it)[8] != '2' ) || ( (*it)[9] != 'A' && (*it)[9] != 'B' ) ) throw std::logic_error( std::string("Bad value in channel configuration: ") + *it);
			*it = it->substr(8,2);
		}
		return ret;
	}

};

#endif // _PARAMS_H

==> atmos-2.96.7.tar.gz: atmos-2.96.7/src/params.h <==
/*
    $Id: params.h 98 2010-03-01 12:55:46Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PARAMS_H
#define _PARAMS_H

#include <algorithm>
#include <cmath>
#include <string>
#include <exception>
#include <stdexcept>

#include <boost/numeric/ublas/vector.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>

#include <yalpa/spheric.h>
#include <yalpa/io.h>

using namespace boost::numeric;
// use ublas::vector for ublas vector and std::vector for std vector

/**
  @brief Container for handling parameters extracted from MASS file.
*/
template<class ContainerType> class parameters_container {
public:
	typedef ContainerType storage_type;
	typedef typename storage_type::key_type   key_type;
	typedef typename storage_type::value_type value_type;
	typedef typename storage_type::const_iterator const_iterator;
private:
	storage_type m_storage;
public:
	parameters_container() {}
	~parameters_container() {}

	bool key_equal( const key_type& k1, const key_type& k2 ) {
		return !m_storage.key_comp()( k1, k2 ) && !m_storage.key_comp()( k2, k1 );
	}
	bool value_equal( const value_type& v1, const value_type& v2 ){
		return !m_storage.value_comp()( v1, v2 ) && !m_storage.value_comp()( v2, v1 ) ;
	}

	bool has(const typename storage_type::key_type& key) const {
		typename storage_type::const_iterator it = m_storage.find( key );
		return ( it != m_storage.end() );
	}
	const typename storage_type::mapped_type& at(const typename storage_type::key_type& key) const {
		typename storage_type::const_iterator it = m_storage.find( key );
		if( it != m_storage.end() )
			return it->second;
		throw std::range_error( std::string("Not found value for key \"") + key + std::string("\""));
	}
	typename storage_type::mapped_type& operator[] (const typename storage_type::key_type& key) {
		return m_storage[key];
	}
	template<class Y> Y as(const typename storage_type::key_type& key) const {
		return boost::lexical_cast<Y>( at(key) );
	}
	template<class Y> Y as(const typename storage_type::key_type& key, const Y& def ) const {
		if( has( key ) )
			return boost::lexical_cast<Y>( at(key) );
		return def;
	}

	const_iterator begin() const { return m_storage.begin(); }
	const_iterator end() const { return m_storage.end(); }

};


/**
  @brief Proxy for access to parameters extracted from MASS file and stored in container.
*/
template<class ParamType> class parameters_proxy {
public:
	typedef ParamType params_type;
private:
	const params_type& m_container;
public:
	parameters_proxy( const params_type& container ): m_container( container ) {}
	~parameters_proxy() {}

	std::string sitename() const { return m_container.at("general/site/sitename"); }
	double longitude() const {
		double longitude = 0.0;
		std::istringstream iss( m_container.at("general/site/longitude") );
		iss.exceptions(std::ios_base::badbit | std::ios_base::failbit);
		iss >> std::sexagesimal >> std::hours >> yalpa::make_angle( longitude );
		return longitude;
	}
	double latitude() const {
		double latitude = 0.0;
		std::istringstream iss( m_container.at("general/site/latitude") );
		iss.exceptions(std::ios_base::badbit | std::ios_base::failbit);
		iss >> std::sexagesimal >> std::degrees >> yalpa::make_angle( latitude );
		return latitude;
	}
	yalpa::geographic<double> site() const {
		return yalpa::geographic<double>( longitude(), latitude() );
	}

	double accumtime()     const { return m_container.template as<double>("operations/normal mode/accumtime"); }
	double basetime()      const { return m_container.template as<double>("operations/normal mode/basetime"); }
	double exposition()    const { return m_container.template as<double>("operations/normal mode/exposition"); }

	double magnification() const { return m_container.template as<double>("optics/common/magnification"); }

	//FIXME: make function with wildcards
	ublas::vector<double> inner_diam()    const {
		ublas::vector<double> ret(4);
		ret(0) = m_container.template as<double>("optics/segment a/inner");
		ret(1) = m_container.template as<double>("optics/segment b/inner");
		ret(2) = m_container.template as<double>("optics/segment c/inner");
		ret(3) = m_container.template as<double>("optics/segment d/inner");
		return ret * ( magnification()*0.1 );
	}
	ublas::vector<double> outer_diam()    const {
		ublas::vector<double> ret(4);
		ret(0) = m_container.template as<double>("optics/segment a/outer");
		ret(1) = m_container.template as<double>("optics/segment b/outer");
		ret(2) = m_container.template as<double>("optics/segment c/outer");
		ret(3) = m_container.template as<double>("optics/segment d/outer");
		return ret * ( magnification()*0.1 );
	}
	ublas::vector<double> nonlinearity() const {
		std::vector< std::string > ctr( counters() );
		ublas::vector<double> ret( ctr.size() );
		for( typename std::vector< std::string >::const_iterator it = ctr.begin(); it != ctr.end(); ++it ) {
			ret( it - ctr.begin() ) = m_container.template as<double>( (boost::format("Devices/Bicounter %1%/NonLinearity%2%") % (*it)[0] % (*it)[1]).str() );
		}
		return ret * 1e-6 / exposition();
	}
	ublas::vector<double> nonpoisson() const {
		std::vector< std::string > ctr( counters() );
		ublas::vector<double> ret( ctr.size() );
		for( typename std::vector< std::string >::const_iterator it = ctr.begin(); it != ctr.end(); ++it ) {
			ret( it - ctr.begin() ) = m_container.template as<double>( (boost::format("Devices/Bicounter %1%/NonPoisson%2%") % (*it)[0] % (*it)[1]).str() );
		}
		return ret;
	}
	ublas::vector<double> scatter() const {
		ublas::vector<double> ret( 4 );
		ret(0) = m_container.template as<double>("channels/scatter/channela",0.0);
		ret(1) = m_container.template as<double>("channels/scatter/channelb",0.0);
		ret(2) = m_container.template as<double>("channels/scatter/channelc",0.0);
		ret(3) = m_container.template as<double>("channels/scatter/channeld",0.0);
		return ret;
	}
	//FIXME: use tuple
	std::vector< std::string > counters() const {
		std::vector< std::string > ret;
		ret.resize(4);
		ret[0] = m_container.at("channels/configuration/channela");
		ret[1] = m_container.at("channels/configuration/channelb");
		ret[2] = m_container.at("channels/configuration/channelc");
		ret[3] = m_container.at("channels/configuration/channeld");
		for( typename std::vector< std::string >::iterator it = ret.begin(); it != ret.end(); ++it ){
			//FIXME: use regex
			if( it->size() < 10 || it->substr(0,7) != "Counter" || ( (*it)[8] != '1' && (*it)[8] != '2' ) || ( (*it)[9] != 'A' && (*it)[9] != 'B' ) ) throw std::logic_error( std::string("Bad value in channel configuration: ") + *it);
			*it = it->substr(8,2);
		}
		return ret;
	}

	double dimm_aperture_base() const { return 0.01 * m_container.template as<double>("general/dimm/aperturebase",20.0); }
	double dimm_aperture_size() const { return 0.01 * m_container.template as<double>("general/dimm/aperturesize",10.0); }
	double dimm_scale() const { return 4.8481368e-06 * m_container.template as<double>("camera/geometry/scale"); }
	double dimm_Ktransversal() const { return 0.358*(1.0*std::pow( dimm_aperture_size(), -1.0/3.0 ) - 0.811*std::pow( dimm_aperture_base(), -1.0/3.0 )); }
	double dimm_Klongitudinal() const { return 0.358*(1.0*std::pow( dimm_aperture_size(), -1.0/3.0 ) - 0.541*std::pow( dimm_aperture_base(), -1.0/3.0 )); }

};

#endif // _PARAMS_H
